@@ -1,25 +1,63 @@
 extern crate proc_macro;
 
-use proc_macro::TokenStream;
-use quote::quote;
-use syn::{parse_macro_input, Data, DeriveInput, Fields, Index};
+use proc_macro::{Span, TokenStream};
+use quote::{quote, ToTokens};
+use syn::{
+    parse_macro_input, Attribute, Data, DeriveInput, Fields, Ident, Index, Lit, Meta, NestedMeta,
+};
 
-#[proc_macro_derive(ReflectIntermediate)]
+#[derive(Debug, Default)]
+struct TypeAttribs {
+    before_patch_change: Option<Ident>,
+    after_patch_change: Option<Ident>,
+}
+
+#[derive(Debug, Default)]
+struct FieldAttribs {
+    pub ignore: bool,
+}
+
+#[proc_macro_derive(ReflectIntermediate, attributes(reflect_intermediate))]
 pub fn derive_reflect_intermediate(input: TokenStream) -> TokenStream {
     let ast = parse_macro_input!(input as DeriveInput);
+    let attribs = parse_type_attribs(&ast.attrs);
+    let before_patch_change = match attribs.before_patch_change {
+        Some(name) => {
+            quote! {
+                fn before_patch_change(&mut self) {
+                    self.#name();
+                }
+            }
+        }
+        None => Default::default(),
+    };
+    let after_patch_change = match attribs.after_patch_change {
+        Some(name) => {
+            quote! {
+                fn after_patch_change(&mut self) {
+                    self.#name();
+                }
+            }
+        }
+        None => Default::default(),
+    };
     let name = &ast.ident;
     let (impl_generics, ty_generics, where_clause) = ast.generics.split_for_impl();
     match ast.data {
         Data::Struct(data) => match data.fields {
             Fields::Named(fields) => {
-                let fields = fields.named.iter().map(|field| {
+                let fields = fields.named.iter().filter_map(|field| {
+                    let attribs = parse_field_attribs(&field.attrs);
+                    if attribs.ignore {
+                        return None;
+                    }
                     let name = field.ident.as_ref().unwrap();
                     let key = name.to_string();
-                    quote! {
+                    Some(quote! {
                         #key => {
                             self.#name.patch_change(change);
                         }
-                    }
+                    })
                 }).collect::<Vec<_>>();
                 quote! {
                     impl #impl_generics serde_reflect_intermediate::ReflectIntermediate for #name #ty_generics #where_clause {
@@ -43,17 +81,25 @@ pub fn derive_reflect_intermediate(input: TokenStream) -> TokenStream {
                             }
                             self.after_patch_change();
                         }
+
+                        #before_patch_change
+
+                        #after_patch_change
                     }
                 }.into()
             }
             Fields::Unnamed(fields) => {
-                let fields = fields.unnamed.iter().enumerate().map(|(index,_)| {
+                let fields = fields.unnamed.iter().enumerate().filter_map(|(index,field)| {
+                    let attribs = parse_field_attribs(&field.attrs);
+                    if attribs.ignore {
+                        return None;
+                    }
                     let tuple_index = Index::from(index);
-                    quote! {
+                    Some(quote! {
                         #index => {
                             self.#tuple_index.patch_change(change);
                         }
-                    }
+                    })
                 }).collect::<Vec<_>>();
                 quote! {
                     impl #impl_generics serde_reflect_intermediate::ReflectIntermediate for #name #ty_generics #where_clause {
@@ -77,6 +123,10 @@ pub fn derive_reflect_intermediate(input: TokenStream) -> TokenStream {
                             }
                             self.after_patch_change();
                         }
+
+                        #before_patch_change
+
+                        #after_patch_change
                     }
                 }.into()
             }
@@ -90,6 +140,11 @@ pub fn derive_reflect_intermediate(input: TokenStream) -> TokenStream {
                 let name = &variant.ident;
                 if let Fields::Unnamed(_) = &variant.fields {
                     if variant.fields.len() == 1 {
+                        let field = variant.fields.iter().next().unwrap();
+                        let attribs = parse_field_attribs(&field.attrs);
+                        if attribs.ignore {
+                            return None;
+                        }
                         Some(quote! {
                             Self::#name(content) => {
                                 content.patch_change(change);
@@ -103,24 +158,38 @@ pub fn derive_reflect_intermediate(input: TokenStream) -> TokenStream {
                 }
             }).collect::<Vec<_>>();
             let struct_variants = data.variants.iter().filter_map(|variant| {
+                let attribs = parse_field_attribs(&variant.attrs);
+                if attribs.ignore {
+                    return None;
+                }
                 let name = &variant.ident;
                 if let Fields::Named(fields) = &variant.fields {
                     let field_names = fields
                         .named
                         .iter()
-                        .map(|field| field.ident.as_ref().unwrap())
+                        .filter_map(|field| {
+                            let attribs = parse_field_attribs(&field.attrs);
+                            if attribs.ignore {
+                                return None;
+                            }
+                            Some(field.ident.as_ref().unwrap())
+                        })
                         .collect::<Vec<_>>();
-                    let fields = fields.named.iter().map(|field| {
+                    let fields = fields.named.iter().filter_map(|field| {
+                        let attribs = parse_field_attribs(&field.attrs);
+                        if attribs.ignore {
+                            return None;
+                        }
                         let name = field.ident.as_ref().unwrap();
                         let key = name.to_string();
-                        quote! {
+                        Some(quote! {
                             #key => {
                                 #name.patch_change(change);
                             }
-                        }
+                        })
                     }).collect::<Vec<_>>();
                     Some(quote! {
-                        Self::#name { #( #field_names ),* } => {
+                        Self::#name { #( #field_names , )* .. } => {
                             for (name, change) in v {
                                 match name.as_str() {
                                     #( #fields )*
@@ -159,9 +228,73 @@ pub fn derive_reflect_intermediate(input: TokenStream) -> TokenStream {
                         }
                         self.after_patch_change();
                     }
+
+                    #before_patch_change
+
+                    #after_patch_change
                 }
             }.into()
         }
         _ => panic!("ReflectIntermediate can be derived only for structs and enums"),
     }
 }
+
+fn parse_type_attribs(attrs: &[Attribute]) -> TypeAttribs {
+    let mut result = TypeAttribs::default();
+    for attrib in attrs {
+        match attrib.parse_meta() {
+            Err(error) => panic!(
+                "Could not parse attribute `{}`: {:?}",
+                attrib.to_token_stream(),
+                error
+            ),
+            Ok(Meta::List(meta)) => {
+                if meta.path.is_ident("reflect_intermediate") {
+                    for meta in meta.nested {
+                        if let NestedMeta::Meta(Meta::NameValue(meta)) = &meta {
+                            if meta.path.is_ident("before_patch_change") {
+                                if let Lit::Str(value) = &meta.lit {
+                                    result.before_patch_change =
+                                        Some(Ident::new(&value.value(), Span::call_site().into()));
+                                }
+                            } else if meta.path.is_ident("after_patch_change") {
+                                if let Lit::Str(value) = &meta.lit {
+                                    result.after_patch_change =
+                                        Some(Ident::new(&value.value(), Span::call_site().into()));
+                                }
+                            }
+                        }
+                    }
+                }
+            }
+            _ => {}
+        }
+    }
+    result
+}
+
+fn parse_field_attribs(attrs: &[Attribute]) -> FieldAttribs {
+    let mut result = FieldAttribs::default();
+    for attrib in attrs {
+        match attrib.parse_meta() {
+            Err(error) => panic!(
+                "Could not parse attribute `{}`: {:?}",
+                attrib.to_token_stream(),
+                error
+            ),
+            Ok(Meta::List(meta)) => {
+                if meta.path.is_ident("reflect_intermediate") {
+                    for meta in meta.nested {
+                        if let NestedMeta::Meta(Meta::Path(path)) = &meta {
+                            if path.is_ident("ignore") {
+                                result.ignore = true;
+                            }
+                        }
+                    }
+                }
+            }
+            _ => {}
+        }
+    }
+    result
+}
